// project created on 06.08.2002 at 19:45
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;

namespace BubbleGame
{

class Matrix : Object
{			
	const int COLORS = 10;
	const int SIZE = 40;
	public bool rearrange;
	
	Random rnd = new Random((int)DateTime.Now.Ticks);	
	ArrayList tiles = new ArrayList();		
	Tile MarkedTile1, MarkedTile2;
	
	public Matrix()
	{			
		for(int x=0; x<8; x++)
			for(int y=-8; y<8; y++)
			{
				Tile tile = new Tile(x * SIZE, y * SIZE);			
				int colorIndex = rnd.Next(COLORS);
				tile.SetColor(colorIndex);
				tiles.Add(tile);
			}
		
		Shuffle();
	}
	
	public void Shuffle()
	{		
		while(IsShuffled() == false)			
			foreach(Tile t1 in tiles)			
				foreach(Tile t2 in tiles)				
					if((t2.IsEastNeighbor(t1) || t2.IsSouthNeighbor(t1)) && t2.SameColor(t1))				
						t2.SetColor(rnd.Next(COLORS));
	}
	
	bool IsShuffled()
	{			
		foreach(Tile t1 in tiles)		
			foreach(Tile t2 in tiles)		
				if((t2.IsEastNeighbor(t1) || t2.IsSouthNeighbor(t1)) && t2.SameColor(t1))			
					return false;
				
		return true;
	}
	
	public void MarkTiles(Point p)
	{			
		foreach(Tile t1 in tiles)				
			foreach(Tile t2 in tiles)				
			{	
				if(t1.Alive && t2.Alive && t1.Settled && t1.MouseInside(p))
				{
					if(t1.HoverOver(p) == 1) //hover in the West
					{					
						t2.Unmark();
						
						if(HasWestNeighbor(t1))
							t1.Mark();
						
						if(t2.IsWestNeighbor(t1))
						{
							t2.Mark();	
							MarkedTile1 = t1;
							MarkedTile2 = t2;						
						}
					}
					else if(t1.HoverOver(p) == 2) //hover in the North
					{
						t2.Unmark();
						
						if(HasNorthNeighbor(t1))
							t1.Mark();
						
						if(t2.IsNorthNeighbor(t1))
						{
							t2.Mark();	
							MarkedTile1 = t1;
							MarkedTile2 = t2;						
						}
					}
					else if(t1.HoverOver(p) == 3) //hover in the East
					{
						t2.Unmark();
						
						if(t1.X < SIZE * 7 && HasEastNeighbor(t1))
							t1.Mark();
						
						if(t2.IsEastNeighbor(t1))
						{
							t2.Mark();
							MarkedTile1 = t1;
							MarkedTile2 = t2;						
						}					
					}
					else if(t1.HoverOver(p) == 4) //hover in the South
					{
						t2.Unmark();
						
						if(t1.Y < SIZE * 7)
							t1.Mark();
						
						if(t2.IsSouthNeighbor(t1))
						{
							t2.Marked = true;	
							MarkedTile1 = t1;
							MarkedTile2 = t2;					
						}
					}	
				}			
		}				
	}
	
	bool HasNorthNeighbor(Tile tile)
	{
		foreach(Tile t in tiles)
			if(t.IsNorthNeighbor(tile) && t.Alive)
						return true;		
				
		return false;
	}
	
	bool HasWestNeighbor(Tile tile)
	{
		foreach(Tile t in tiles)
			if(t.IsWestNeighbor(tile) && t.Alive)
						return true;		
				
		return false;
	}
	
	bool HasEastNeighbor(Tile tile)
	{
		foreach(Tile t in tiles)
			if(t.IsEastNeighbor(tile) && t.Alive)
						return true;		
				
		return false;
	}
	
	public void SwitchTiles()
	{			
		Tile tempTile = MarkedTile1;
		
		foreach(Tile t in tiles)	
		{
			if(t.Marked && t.Alive && MarkedTile1.Fading == false && MarkedTile2.Fading == false)
			{
				if(t == MarkedTile1)
				{
					t.Switching = true;
					t.MoveTo(MarkedTile2.X, MarkedTile2.Y);
				}
				if(t == MarkedTile2)
				{
					t.Switching = true;
					t.MoveTo(tempTile.X, tempTile.Y);	
				}
			}
		}			
	}
	
	void RemoveTiles()
	{
		foreach(Tile t1 in tiles)		
			foreach(Tile t2 in tiles)			
				if((t2.IsEastNeighbor(t1) || t2.IsSouthNeighbor(t1)) && t2.SameColor(t1))
				{
					if(t1.Settled && t2.Settled)
					{
						t1.Destroy();					
						t2.Destroy();		
					}
				}		
	}
	
	public void RearrangeTiles()
	{		
		foreach(Tile t in tiles)					
			t.MoveTo(t.X, BottomOf(t));
	}
	
	bool Switching()
	{
		foreach(Tile t in tiles)
			if(t.Switching)
				return true;
		
		return false;
	}	
	
	int BottomOf(Tile t1)
	{
		int maxY = 8 * SIZE;
		int max = maxY;
			
		foreach(Tile t2 in tiles)	
			if(t2.X == t1.X && t2.Y > t1.Y && t2.Alive && t2.Y < max)
				max = t2.Y;
		
		if(max < maxY)
			return max - SIZE;
			
		return maxY - SIZE;		
	}
	
	public void UnmarkTiles()
	{
		foreach(Tile t in tiles)
			t.Marked = false;
	}
	
	public int TileNumber()
	{
		int number = 0;
		
		foreach(Tile t in tiles)
			if(t.Alive)
				number ++;
		
		return number;
	}

	public void Draw(Graphics g)  
	{						
		RemoveTiles();	
				
		if(Switching() == false)
			RearrangeTiles();
	
		foreach(Tile t in tiles)					
			t.Draw(g);				
	} 
}
}
